//
// This program reads a text file and writes the tokens in another. 
// It cleans the tokens, and removes HTLM tags.
// works in java 1.1.7 or higher
// written by Nick Evaggelopoulos
//
// The filenames of input and output file are entered by the user at the command line.
//

import java.io.*;
import java.lang.*;
import java.util.*;
import java.lang.reflect.*;


public class CleanHTML2{
       public static void main(String[] args) throws FileNotFoundException,IOException{
            String infile = new String();
            String outfile = new String(); 
            
                 //Check the number of arguments
		 if(Array.getLength(args) < 2) {
                      System.out.println("Usage: CleanHTML2 <input file> <output file>");
                      System.exit(0);
		 }
                 infile = args[0];  // hardcoded filename, not a good programming style
                 outfile = args[1];  // hardcoded filename, not a good programming style
                 FileReader fr = new FileReader(infile);
                 BufferedReader br = new BufferedReader(fr);
                 CleanHTML2 c = new CleanHTML2();               // class constructor
                 Vector lineVec =  c.readline(br);
                 fr.close();

                 System.out.println(lineVec);   // for testing purposes
                 int pos = 0;
                 Vector tokenVec = c.getTokens(lineVec);

                 // this chunk of code writes the tokens to a file
 
                 FileOutputStream fos = new FileOutputStream(outfile);
                 DataOutputStream dos = new DataOutputStream(fos);

                 int index = 0;
                 while(index < tokenVec.size()){ 
                      dos.writeBytes((String)tokenVec.elementAt(index) + "\n");
                      index++;
                  }  
       } // end main


       // the following function readline takes a BufferedReader as an argument
       // and reads a text file. Each line is read into a vector.

       public Vector readline(BufferedReader br) throws IOException{
              Vector linevec = new Vector();
              String line;
              int c;
              char[] cbuf = new char[1];    // "character buffer" cbuf is an array of characters
              int j = 0;
              while((c = br.read(cbuf)) != -1){
                     line = new String(cbuf,0,1)+br.readLine();
                     linevec.insertElementAt(line,j);           
                     j++;
              } 
             
              return linevec;
       }            
   

       //  The following function takes a vector of lines of text and 
       //  returns a vector that consists of tokens with clean text,
       // after removal of HTML tags
    
       public Vector getTokens(Vector v){
          int line = 0;
          int len = 0;
          int tagStart, tagEnd = 0;
          int tagFlag = 0;
          Vector tokenVec = new Vector();
          String s = new String();
          String ts = new String();
          while(line < v.size()){
             s = (String) v.elementAt(line);
             s = s.toLowerCase();

             // remove HTML tags             
             int exitFlag = 0;
             while(exitFlag == 0){
                if(tagFlag == 1) tagStart = 0;   // if line start is already HTML
                else tagStart = s.indexOf('<', 0);
                if(tagStart == -1) tagEnd = -1;
                else tagEnd = s.indexOf('>', tagStart);
                if(tagEnd == -1){        // if no end tag in the line
                   if(tagStart > -1){    // if line has some HTML
                      tagFlag = 1;    
                      tagEnd = s.length() - 1;
                   }
                   exitFlag = 1;
                }
                else tagFlag = 0;
                if ((tagStart == -1) && (tagEnd == -1)) exitFlag = 1;
                else{
                   s = s + " ";
                   ts = s.substring(tagStart, tagEnd + 1);
                   if(ts.indexOf('<') > -1) tagStart = tagStart + ts.indexOf('<');
                   s = s.substring(0, tagStart) + " " + s.substring(tagEnd + 1, s.length());
                }
             }

             // remove special characters
             s = s.replace(';',' ');
             s = s.replace(',',' ');
             s = s.replace('?',' ');
             s = s.replace('!',' ');
             s = s.replace(':',' ');
             s = s.replace('>',' ');
             s = s.replace('<',' ');
             s = s.replace('.',' ');
             s = s.replace('=',' ');
             s = s.replace('/',' '); 
             s = s.replace('$',' ');
             s = s.replace('&',' ');
             s = s.replace('@',' ');
             s = s.replace('%',' ');
             s = s.replace('#',' ');
             s = s.replace('*',' ');
             s = s.replace('"',' ');
             s = s.replace('-',' ');
             s = s.replace('_',' ');
             s = s.replace('\'',' ');
             s = s.replace('+',' ');
             s = s.replace('|',' ');
             s = s.replace('(',' ');
             s = s.replace(')',' ');
             s = s.replace('[',' ');
             s = s.replace(']',' ');

     
             StringTokenizer st = new StringTokenizer(s);

             // remove special tokens (starting with a number or 
             // having length of one character)

             while(st.hasMoreTokens()){
                String st2 = new String();
                st2 = st.nextToken();
                StringBuffer sb2 = new StringBuffer(st2);
                if(!(Character.isDigit(sb2.charAt(0))) && (st2.length()) > 1){
                              tokenVec.addElement(st2);
                }
             }
             line++;
          }    
       return tokenVec;
       } // end getTokens   
}// end class
          
 

